/* timezone.c
 * Copyright (C) 2007-2008 Gauvain Pocentek <gpocentek@linutop.com>
 * This file is part of the linutop config editor
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the license.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "main.h"
#include "timezone.h"
#include "utils.h"

/* globale */
gchar        **split;
GtkTreeStore  *treestore1;
GtkTreeStore  *treestore2;
GtkWidget     *treeview1;
GtkWidget     *treeview2;


GtkTreeIter
populate_continent_tree (GtkTreeStore * treestore)
{
    int i = 0;
    gboolean found = FALSE;
    GtkTreeIter iter;
    GtkTreeIter ret;

    while (tz_top[i] != '\0') {
        gtk_tree_store_append (treestore, &iter, NULL);
        gtk_tree_store_set (treestore, &iter, 0, tz_top[i], 1, i, -1);
        if (strcmp (split[0], tz_top[i]) == 0) {
            found = TRUE;
            ret = iter;
        }
        i++;
    }
    if (!found)
        gtk_tree_model_get_iter_first (GTK_TREE_MODEL (treestore), &ret);
    return ret;
}

static void
update_country (GtkTreeView * widget, gpointer data)
{
    int continent;
    int i = 0;
    gboolean found = FALSE;
    GtkTreeIter seliter;
    GtkTreeIter iter;
    GtkTreeModel *model;
    GtkTreeSelection *selection;
    GtkTreePath *path;

    selection = gtk_tree_view_get_selection (widget);
    model = gtk_tree_view_get_model (widget);
    gtk_tree_selection_get_selected (selection, &model, &iter);
    gtk_tree_model_get (model, &iter, 1, &continent, -1);

    gtk_tree_store_clear (GTK_TREE_STORE (treestore2));

    while (tz_list[continent][i] != '\0') {
        gtk_tree_store_append (GTK_TREE_STORE (treestore2),
                      &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (treestore2), &iter,
                   0, tz_list[continent][i], -1);
        if (strcmp (split[1], tz_list[continent][i]) == 0) {
            found = TRUE;
            seliter = iter;
        }
        i++;
    }
    selection =
        gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview2));
    if (!found)
        gtk_tree_model_get_iter_first (GTK_TREE_MODEL (treestore2),
                      &seliter);
    gtk_tree_selection_select_iter (selection, &seliter);
    path = gtk_tree_model_get_path (GTK_TREE_MODEL (treestore2), &iter);
    gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (treeview2), path,
                                  NULL, TRUE, 0.5, 0.5);
    gtk_tree_path_free (path);
}

void
tz_dialog_run ()
{
    int id;
    GtkWidget *new_dialog;
    treeview1 = gtk_tree_view_new ();
    treeview2 = gtk_tree_view_new ();
    GtkCellRenderer *renderer;
    GtkTreeSelection *selection;
    GtkTreePath *path;
    GtkTreeIter iter;
    GtkTreeModel *model;
    GtkWidget *scw1, *scw2;
    GtkWidget *hbox;
    gchar *code = g_strdup (get_tz_value ());
    split = g_strsplit (code, "/", 2);
    g_free (code);

    new_dialog =
        xfce_titled_dialog_new_with_buttons (_("Linutop - Time zone"),
                        GTK_WINDOW (dialog->dialog),
                        GTK_DIALOG_MODAL |
                        GTK_DIALOG_DESTROY_WITH_PARENT,
                        GTK_STOCK_CANCEL,
                        GTK_RESPONSE_CANCEL,
                        GTK_STOCK_OK, GTK_RESPONSE_OK,
                        NULL);
    gtk_window_set_icon_name (GTK_WINDOW (new_dialog), "config-date");
    xfce_titled_dialog_set_subtitle ((XfceTitledDialog *) new_dialog,
                    _("Select a time zone"));

    hbox = gtk_hbox_new (TRUE, 6);
    gtk_container_add (GTK_CONTAINER (GTK_DIALOG (new_dialog)->vbox), hbox);

    scw1 = gtk_scrolled_window_new (NULL, NULL);
    gtk_box_pack_start (GTK_BOX (hbox), scw1, TRUE, TRUE, 2);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scw1),
                       GTK_POLICY_AUTOMATIC,
                       GTK_POLICY_AUTOMATIC);
    scw2 = gtk_scrolled_window_new (NULL, NULL);
    gtk_box_pack_start (GTK_BOX (hbox), scw2, TRUE, TRUE, 2);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scw2),
                       GTK_POLICY_AUTOMATIC,
                       GTK_POLICY_AUTOMATIC);

    /* Continent tree */
    treestore1 = gtk_tree_store_new (2, G_TYPE_STRING, G_TYPE_INT);
    iter = populate_continent_tree (GTK_TREE_STORE (treestore1));
    renderer = gtk_cell_renderer_text_new ();

    gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW
                            (treeview1), -1,
                            "Continent", renderer,
                            "text", 0, NULL);
    gtk_tree_view_set_model (GTK_TREE_VIEW (treeview1),
                GTK_TREE_MODEL (treestore1));
    selection =
        gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview1));
    gtk_tree_selection_select_iter (selection, &iter);
    path = gtk_tree_model_get_path (GTK_TREE_MODEL (treestore1), &iter);
    gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (treeview1), path,
                                 NULL, TRUE, 0.5, 0.5);
    gtk_tree_path_free (path);
    gtk_container_add (GTK_CONTAINER (scw1), treeview1);

    /* City/Country tree */
    treestore2 = gtk_tree_store_new (1, G_TYPE_STRING);

    model = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview1));
    selection =
        gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview1));
    gtk_tree_selection_get_selected (selection, &model, &iter);

    gtk_tree_model_get (model, &iter, 1, &id, -1);

    renderer = gtk_cell_renderer_text_new ();

    gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW
                            (treeview2), -1,
                            "City/Country", renderer,
                            "text", 0, NULL);
    gtk_tree_view_set_model (GTK_TREE_VIEW (treeview2),
                GTK_TREE_MODEL (treestore2));
    gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE
                         (treestore2), 0,
                         GTK_SORT_ASCENDING);
    gtk_container_add (GTK_CONTAINER (scw2), treeview2);

    g_signal_connect (G_OBJECT (treeview1), "cursor-changed",
             G_CALLBACK (update_country), NULL);
    g_signal_connect_swapped (G_OBJECT (treeview2), "row-activated",
             G_CALLBACK (cb_row_activated), new_dialog);

    gtk_widget_show_all (hbox);

    gtk_window_set_default_size (GTK_WINDOW (new_dialog), 400, 400);

    int response = gtk_dialog_run (GTK_DIALOG (new_dialog));
    g_strfreev (split);
    if (response == GTK_RESPONSE_OK) {
        gchar *tmp = NULL, *timez = NULL;
        selection =
            gtk_tree_view_get_selection (GTK_TREE_VIEW
                         (treeview1));
        gtk_tree_selection_get_selected (selection, &model, &iter);
        gtk_tree_model_get (model, &iter, 0, &timez, -1);

        selection =
            gtk_tree_view_get_selection (GTK_TREE_VIEW
                         (treeview2));
        gtk_tree_selection_get_selected (selection, &model, &iter);
        gtk_tree_model_get (model, &iter, 0, &tmp, -1);
        timez = g_strconcat (timez, "/", tmp, NULL);
        set_tz (timez);
        set_tz_user_defined (TRUE);
        g_free (tmp);
        g_free (timez);
    }
    gtk_widget_destroy (new_dialog);
}
